package java.awt;

import java.text.BreakIterator;
import javax.swing.text.AttributeSet;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.security.util.SecurityConstants;
import java.awt.event.TextEvent;
import java.util.EventListener;
import java.awt.peer.TextComponentPeer;
import java.awt.im.InputMethodRequests;
import sun.awt.InputMethodSupport;
import java.awt.event.TextListener;
import javax.accessibility.Accessible;

public class TextComponent extends Component implements Accessible
{
    String text;
    boolean editable;
    int selectionStart;
    int selectionEnd;
    boolean backgroundSetByClientCode;
    protected transient TextListener textListener;
    private static final long serialVersionUID = -2214773872412987419L;
    private int textComponentSerializedDataVersion;
    private boolean checkForEnableIM;
    
    TextComponent(String s) throws HeadlessException {
        this.editable = true;
        this.backgroundSetByClientCode = false;
        this.textComponentSerializedDataVersion = 1;
        this.checkForEnableIM = true;
        GraphicsEnvironment.checkHeadless();
        this.text = ((s != null) ? s : "");
        this.setCursor(Cursor.getPredefinedCursor(2));
    }
    
    private void enableInputMethodsIfNecessary() {
        if (this.checkForEnableIM) {
            this.checkForEnableIM = false;
            try {
                Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
                boolean enableInputMethodsForTextComponent = false;
                if ((defaultToolkit instanceof InputMethodSupport)) {
                    enableInputMethodsForTextComponent = ((InputMethodSupport)defaultToolkit).enableInputMethodsForTextComponent();
                }
                this.enableInputMethods((boolean)enableInputMethodsForTextComponent);
            }
            catch (Exception ex) {}
        }
    }
    
    @Override
    public void enableInputMethods(boolean b) {
        this.checkForEnableIM = false;
        super.enableInputMethods(b);
    }
    
    @Override
    boolean areInputMethodsEnabled() {
        if (this.checkForEnableIM) {
            this.enableInputMethodsIfNecessary();
        }
        return (this.eventMask & 0x1000L) != 0x0L;
    }
    
    @Override
    public InputMethodRequests getInputMethodRequests() {
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            return textComponentPeer.getInputMethodRequests();
        }
        return null;
    }
    
    @Override
    public void addNotify() {
        super.addNotify();
        this.enableInputMethodsIfNecessary();
    }
    
    @Override
    public void removeNotify() {
        synchronized (this.getTreeLock()) {
            TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
            if (textComponentPeer != null) {
                this.text = textComponentPeer.getText();
                this.selectionStart = textComponentPeer.getSelectionStart();
                this.selectionEnd = textComponentPeer.getSelectionEnd();
            }
            super.removeNotify();
        }
    }
    
    public synchronized void setText(String s) {
        int n = (this.text == null || this.text.isEmpty()) && (s == null || s.isEmpty());
        this.text = ((s != null) ? s : "");
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null && n == 0) {
            textComponentPeer.setText(this.text);
        }
    }
    
    public synchronized String getText() {
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            this.text = textComponentPeer.getText();
        }
        return this.text;
    }
    
    public synchronized String getSelectedText() {
        return this.getText().substring(this.getSelectionStart(), this.getSelectionEnd());
    }
    
    public boolean isEditable() {
        return this.editable;
    }
    
    public synchronized void setEditable(boolean b) {
        if (this.editable == b) {
            return;
        }
        this.editable = b;
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            textComponentPeer.setEditable(b);
        }
        return;
    }
    
    @Override
    public Color getBackground() {
        if (!this.editable && !this.backgroundSetByClientCode) {
            return SystemColor.control;
        }
        return super.getBackground();
    }
    
    @Override
    public void setBackground(Color background) {
        this.backgroundSetByClientCode = true;
        super.setBackground(background);
    }
    
    public synchronized int getSelectionStart() {
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            this.selectionStart = textComponentPeer.getSelectionStart();
        }
        return this.selectionStart;
    }
    
    public synchronized void setSelectionStart(int n) {
        this.select(n, this.getSelectionEnd());
    }
    
    public synchronized int getSelectionEnd() {
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            this.selectionEnd = textComponentPeer.getSelectionEnd();
        }
        return this.selectionEnd;
    }
    
    public synchronized void setSelectionEnd(int n) {
        this.select(this.getSelectionStart(), n);
    }
    
    public synchronized void select(int length, int length2) {
        String text = this.getText();
        if (length < 0) {
            length = 0;
        }
        if (length > text.length()) {
            length = text.length();
        }
        if (length2 > text.length()) {
            length2 = text.length();
        }
        if (length2 < length) {
            length2 = length;
        }
        this.selectionStart = length;
        this.selectionEnd = length2;
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            textComponentPeer.select(length, length2);
        }
    }
    
    public synchronized void selectAll() {
        this.selectionStart = 0;
        this.selectionEnd = this.getText().length();
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            textComponentPeer.select(this.selectionStart, this.selectionEnd);
        }
    }
    
    public synchronized void setCaretPosition(int caretPosition) {
        if (caretPosition < 0) {
            throw new IllegalArgumentException("position less than zero.");
        }
        int length = this.getText().length();
        if (caretPosition > length) {
            caretPosition = length;
        }
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            textComponentPeer.setCaretPosition(caretPosition);
        }
        else {
            this.select(caretPosition, caretPosition);
        }
        return;
    }
    
    public synchronized int getCaretPosition() {
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        int n;
        if (textComponentPeer != null) {
            n = textComponentPeer.getCaretPosition();
        }
        else {
            n = this.selectionStart;
        }
        int length = this.getText().length();
        if (n > length) {
            n = length;
        }
        return n;
    }
    
    public synchronized void addTextListener(TextListener textListener) {
        if (textListener == null) {
            return;
        }
        this.textListener = AWTEventMulticaster.add(this.textListener, textListener);
        this.newEventsOnly = true;
        return;
    }
    
    public synchronized void removeTextListener(TextListener textListener) {
        if (textListener == null) {
            return;
        }
        this.textListener = AWTEventMulticaster.remove(this.textListener, textListener);
        return;
    }
    
    public synchronized TextListener[] getTextListeners() {
        return (TextListener[])this.getListeners(TextListener.class);
    }
    
    @Override
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        if (clazz == TextListener.class) {
            return AWTEventMulticaster.getListeners(this.textListener, clazz);
        }
        return super.getListeners(clazz);
    }
    
    @Override
    boolean eventEnabled(AWTEvent awtEvent) {
        if (awtEvent.id == 900) {
            return (this.eventMask & 0x400L) != 0x0L || this.textListener != null;
        }
        return super.eventEnabled(awtEvent);
    }
    
    @Override
    protected void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof TextEvent)) {
            this.processTextEvent((TextEvent)awtEvent);
            return;
        }
        super.processEvent(awtEvent);
        return;
    }
    
    protected void processTextEvent(TextEvent textEvent) {
        TextListener textListener = this.textListener;
        if (textListener != null) {
            switch (textEvent.getID()) {
                case 900: {
                    textListener.textValueChanged(textEvent);
                    break;
                }
            }
        }
    }
    
    @Override
    protected String paramString() {
        String s = super.paramString() + ",text=" + this.getText();
        if (this.editable) {
            s += ",editable";
        }
        return s + ",selection=" + this.getSelectionStart() + "-" + this.getSelectionEnd();
    }
    
    private boolean canAccessClipboard() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null) {
            return true;
        }
        try {
            securityManager.checkPermission(SecurityConstants.AWT.ACCESS_CLIPBOARD_PERMISSION);
            return true;
        }
        catch (SecurityException ex) {
            return false;
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        TextComponentPeer textComponentPeer = (TextComponentPeer)this.peer;
        if (textComponentPeer != null) {
            this.text = textComponentPeer.getText();
            this.selectionStart = textComponentPeer.getSelectionStart();
            this.selectionEnd = textComponentPeer.getSelectionEnd();
        }
        objectOutputStream.defaultWriteObject();
        AWTEventMulticaster.save(objectOutputStream, "textL", this.textListener);
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        GraphicsEnvironment.checkHeadless();
        objectInputStream.defaultReadObject();
        this.text = ((this.text != null) ? this.text : "");
        this.select(this.selectionStart, this.selectionEnd);
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            if ("textL" == ((String)object).intern()) {
                this.addTextListener((TextListener)(TextListener)objectInputStream.readObject());
            }
            else {
                objectInputStream.readObject();
            }
        }
        this.enableInputMethodsIfNecessary();
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new TextComponent.AccessibleAWTTextComponent();
        }
        return this.accessibleContext;
    }
    
    protected class AccessibleAWTTextComponent extends Component.AccessibleAWTComponent implements AccessibleText, TextListener
    {
        private static final long serialVersionUID = 3631432373506317811L;
        private static final boolean NEXT = true;
        private static final boolean PREVIOUS = false;
        
        public AccessibleAWTTextComponent() {
            TextComponent.this.addTextListener(this);
        }
        
        @Override
        public void textValueChanged(TextEvent textEvent) {
            this.firePropertyChange("AccessibleText", null, Integer.valueOf(TextComponent.this.getCaretPosition()));
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (TextComponent.this.isEditable()) {
                accessibleStateSet.add(AccessibleState.EDITABLE);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TEXT;
        }
        
        @Override
        public AccessibleText getAccessibleText() {
            return this;
        }
        
        @Override
        public int getIndexAtPoint(Point point) {
            return -1;
        }
        
        @Override
        public Rectangle getCharacterBounds(int n) {
            return null;
        }
        
        @Override
        public int getCharCount() {
            return TextComponent.this.getText().length();
        }
        
        @Override
        public int getCaretPosition() {
            return TextComponent.this.getCaretPosition();
        }
        
        @Override
        public AttributeSet getCharacterAttribute(int n) {
            return null;
        }
        
        @Override
        public int getSelectionStart() {
            return TextComponent.this.getSelectionStart();
        }
        
        @Override
        public int getSelectionEnd() {
            return TextComponent.this.getSelectionEnd();
        }
        
        @Override
        public String getSelectedText() {
            String selectedText = TextComponent.this.getSelectedText();
            if (selectedText == null || selectedText.equals("")) {
                return null;
            }
            return selectedText;
        }
        
        @Override
        public String getAtIndex(int n, int beginIndex) {
            if (beginIndex < 0 || beginIndex >= TextComponent.this.getText().length()) {
                return null;
            }
            switch (n) {
                case 1: {
                    return TextComponent.this.getText().substring(beginIndex, beginIndex + 1);
                }
                case 2: {
                    String text = TextComponent.this.getText();
                    BreakIterator wordInstance = BreakIterator.getWordInstance();
                    wordInstance.setText(text);
                    return text.substring(wordInstance.previous(), wordInstance.following(beginIndex));
                }
                case 3: {
                    String text2 = TextComponent.this.getText();
                    BreakIterator sentenceInstance = BreakIterator.getSentenceInstance();
                    sentenceInstance.setText(text2);
                    return text2.substring(sentenceInstance.previous(), sentenceInstance.following(beginIndex));
                }
                default: {
                    return null;
                }
            }
        }
        
        private int findWordLimit(int offset, BreakIterator breakIterator, boolean b, String s) {
            int n = b ? breakIterator.following(offset) : breakIterator.preceding(offset);
            for (int i = b ? breakIterator.next() : breakIterator.previous(); i != -1; n = i, i = (b ? breakIterator.next() : breakIterator.previous())) {
                int j = Math.min(n, i);
                while (j < Math.max(n, i)) {
                    if (Character.isLetter(s.charAt(j))) {
                        return n;
                    }
                    ++j;
                }
            }
            return -1;
        }
        
        @Override
        public String getAfterIndex(int n, int n2) {
            if (n2 < 0 || n2 >= TextComponent.this.getText().length()) {
                return null;
            }
            switch (n) {
                case 1: {
                    if (n2 + 1 >= TextComponent.this.getText().length()) {
                        return null;
                    }
                    return TextComponent.this.getText().substring(n2 + 1, n2 + 2);
                }
                case 2: {
                    String text = TextComponent.this.getText();
                    BreakIterator wordInstance = BreakIterator.getWordInstance();
                    wordInstance.setText(text);
                    int wordLimit = this.findWordLimit(n2, wordInstance, true, text);
                    if (wordLimit == -1 || wordLimit >= text.length()) {
                        return null;
                    }
                    int following = wordInstance.following(wordLimit);
                    if (following == -1 || following >= text.length()) {
                        return null;
                    }
                    return text.substring(wordLimit, following);
                }
                case 3: {
                    String text2 = TextComponent.this.getText();
                    BreakIterator sentenceInstance = BreakIterator.getSentenceInstance();
                    sentenceInstance.setText(text2);
                    int following2 = sentenceInstance.following(n2);
                    if (following2 == -1 || following2 >= text2.length()) {
                        return null;
                    }
                    int following3 = sentenceInstance.following(following2);
                    if (following3 == -1 || following3 >= text2.length()) {
                        return null;
                    }
                    return text2.substring(following2, following3);
                }
                default: {
                    return null;
                }
            }
        }
        
        @Override
        public String getBeforeIndex(int n, int endIndex) {
            if (endIndex < 0 || endIndex > TextComponent.this.getText().length() - 1) {
                return null;
            }
            switch (n) {
                case 1: {
                    if (endIndex == 0) {
                        return null;
                    }
                    return TextComponent.this.getText().substring(endIndex - 1, endIndex);
                }
                case 2: {
                    String text = TextComponent.this.getText();
                    BreakIterator wordInstance = BreakIterator.getWordInstance();
                    wordInstance.setText(text);
                    int wordLimit = this.findWordLimit(endIndex, wordInstance, false, text);
                    if (wordLimit == -1) {
                        return null;
                    }
                    int preceding = wordInstance.preceding(wordLimit);
                    if (preceding == -1) {
                        return null;
                    }
                    return text.substring(preceding, wordLimit);
                }
                case 3: {
                    String text2 = TextComponent.this.getText();
                    BreakIterator sentenceInstance = BreakIterator.getSentenceInstance();
                    sentenceInstance.setText(text2);
                    sentenceInstance.following(endIndex);
                    int previous = sentenceInstance.previous();
                    int previous2 = sentenceInstance.previous();
                    if (previous2 == -1) {
                        return null;
                    }
                    return text2.substring(previous2, previous);
                }
                default: {
                    return null;
                }
            }
        }
    }
}
